import { IObservable, MethodeResolution, SectionNub, Session } from "jalhyd";

import { FormResultRemous } from "../form-result-remous";
import { FormDefSection } from "../form-def-section";
import { FormComputeCourbeRemous } from "../form-compute-courbe-remous";
import { FieldSet } from "../../fieldset";
import { FormulaireBase } from "./form-base";

export class FormulaireCourbeRemous extends FormulaireBase {

    private _formSection: FormDefSection;

    /**
     * id du select configurant la méthode de résolution
     */
    private _resolveMethSelectId: string;

    constructor() {
        super();
        this._formSection = new FormDefSection(this);
        this._formResult = new FormResultRemous(this);

        // remove obsolete observer set by super()
        this.removeObserver(this._formCompute);
        this._formCompute = new FormComputeCourbeRemous(this, (this._formResult as FormResultRemous));
        // default properties
        this._props["methodeResolution"] = MethodeResolution.Trapezes;
        this._props["varCalc"] = ""; // important
    }

    protected parseOptions(json: {}) {
        super.parseOptions(json);
        this._formSection.parseOptions(json);

        // id du select configurant la méthode de résolution
        this._resolveMethSelectId = this.getOption(json, "methodSelectId");
    }

    public afterParseFieldset(fs: FieldSet) {
        this._formSection.afterParseFieldset(fs);

        // si le FieldSet contient le select de méthode de résolution
        if (this._resolveMethSelectId) {
            const sel = fs.getFormulaireNodeById(this._resolveMethSelectId);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet
                fs.properties.addObserver(this);
            }
        }
    }

    // interface Observer

    update(sender: IObservable, data: any) {

        super.update(sender, data);

        if (sender instanceof FieldSet && data.action === "propertyChange") {
            switch (sender.id) {
                case "fs_section":
                    // replace underlying section without replacing whole Nub
                    const newSect = Session.getInstance().createSection(sender.properties.getPropValue("nodeType"));
                    (this._currentNub as SectionNub).setSection(newSect);
                    // reflect changes in GUI
                    for (const fs of this.allFieldsets) {
                        // show / hide dependent fields
                        fs.updateFields();
                    }
                    this.reset();
                    break;

                case "fs_param_calc":
                case "fs_target_data":
                    this.reset();
                    break;
            }
        }
    }
}
