import { FixedResults } from "../../results/fixed-results";
import { VarResults } from "../../results/var-results";
import { ParamRadioConfig } from "../ngparam";
import { FormResult } from "./form-result";
import { FormulaireDefinition } from "./form-definition";
import { CalculatorResults } from "../../results/calculator-results";
import { GraphType } from "../../results/graph-type";

export class FormResultFixedVar extends FormResult {
    /**
     * résultats fixes/variables
     */
    protected _fixedResults: FixedResults;
    protected _varResults: VarResults;

    protected _formBase: FormulaireDefinition;

    constructor(base: FormulaireDefinition) {
        super();
        this._formBase = base;
        this._fixedResults = new FixedResults();
        this._varResults = new VarResults();
    }

    public get fixedResults() {
        return this._fixedResults;
    }

    public get varResults() {
        return this._varResults;
    }

    public resetResults() {
        this._fixedResults.reset();
        this._varResults.reset();
    }

    public addFixedParameters() {
        for (const p of this._formBase.getDisplayedParamListFromState(ParamRadioConfig.FIX)) {
            this._fixedResults.addFixedParameter(p);
        }

        for (const p of this._formBase.getDisplayedParamListFromState(ParamRadioConfig.LINK)) {
            if (!p.paramDefinition.hasMultipleValues) {
                this._fixedResults.addFixedParameter(p);
            }
        }
    }

    public set graphType(t: GraphType) {
        this._varResults.graphType = t;
    }

    public get hasResults(): boolean {
        return this._fixedResults.hasResults || this._varResults.hasResults;
    }

    public get results(): CalculatorResults[] {
        const res: CalculatorResults[] = [];
        res.push(this._fixedResults);
        res.push(this._varResults);
        return res;
    }
}
